@@ -1,6 +1,6 @@
 /* BSD 3-Clause License
  *
- * Copyright © 2008-2019, Jice and the libtcod contributors.
+ * Copyright © 2008-2020, Jice and the libtcod contributors.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,12 +35,11 @@
 #include <string>
 
 #include "console.h"
-#include "console/drawing.h"
-#include "console/printing.h"
-#include "console/rexpaint.h"
-#include "engine/display.h"
-
-#ifdef TCOD_CONSOLE_SUPPORT
+#include "console_etc.h"
+#include "console_init.h"
+#include "console_drawing.h"
+#include "console_printing.h"
+#include "console_rexpaint.h"
 
 #include "image.hpp"
 #include "color.hpp"
@@ -693,11 +692,12 @@ public :
    *    UTF-8 overloads.
    *  \endrst
    */
-  TCOD_DEPRECATED("Use TCODConsole::printf or the std::string overload for"
-                  " this function.")
+  TCOD_DEPRECATED(
+    "Use TCODConsole::printf or the std::string overload for this function."
+    "\nNote that you'll need to use UTF-8 encoded strings for those functions.")
   void print(int x, int y, const char *fmt, ...);
   /**
-   *  Print a UTF-8 string to the console.
+   *  Print an EASCII encoded string to the console.
    *
    *  This method will use this consoles default alignment, blend mode, and
    *  colors.
@@ -744,8 +744,10 @@ public :
    *    These functions have overloads for specifying flag and alignment.
    *  \endrst
    */
-  TCOD_DEPRECATED("Use TCODConsole::print or TCODConsole::printf instead of"
-                  " this function.")
+  TCOD_DEPRECATED(
+    "Use TCODConsole::print (std::string overload) or TCODConsole::printf"
+    "instead of this function."
+    "\nNote that you'll need to use UTF-8 encoded strings for those functions.")
   void printEx(int x, int y, TCOD_bkgnd_flag_t flag,
                TCOD_alignment_t alignment, const char *fmt, ...);
 	/**
@@ -1036,6 +1038,7 @@ public :
 	@Param fmt if NULL, the function only draws a rectangle.
 		Else, printf-like format string, eventually followed by parameters. You can use control codes to change the colors inside the string.
 	*/
+	TCODLIB_FORMAT(8, 9)
 	void printFrame(int x,int y,int w,int h, bool clear=true, TCOD_bkgnd_flag_t flag = TCOD_BKGND_DEFAULT, const char *fmt=NULL, ...);
 
 	/**
@@ -1799,18 +1802,22 @@ public :
 	static const char *getColorControlString( TCOD_colctrl_t ctrl );
 	// ctrl = TCOD_COLCTRL_FORE_RGB or TCOD_COLCTRL_BACK_RGB
 	static const char *getRGBColorControlString( TCOD_colctrl_t ctrl, const TCODColor & col );
-
+  /**
+   *  Return a pointer to the underlying TCOD_Console struct.
+   *  \rst
+   *  .. versionadded:: 1.14
+   *  \endrst
+   */
+  TCOD_Console* get_data() noexcept
+  {
+    return data;
+  }
+  const TCOD_Console* get_data() const noexcept
+  {
+    return data;
+  }
 protected :
-	friend class TCODImage;
-	friend class TCODZip;
-	friend class TCODText;
-  friend TCODLIB_API void tcod::console::init_root(
-      int w, int h, const std::string& title,
-      bool fullscreen, TCOD_renderer_t renderer);
-	TCODConsole();
-	TCOD_Console* data;
+  TCODConsole();
+  TCOD_Console* data = nullptr;
 };
-
-#endif /* TCOD_CONSOLE_SUPPORT */
-
 #endif /* _TCOD_CONSOLE_HPP */
