@@ -31,7 +31,7 @@ import warnings
 import numpy as np
 
 import tcod.constants
-from tcod.libtcod import ffi, lib
+from tcod.loader import ffi, lib
 import tcod._internal
 from tcod._internal import deprecate
 
@@ -102,9 +102,10 @@ class Console:
     # A structured arrays type with the added "fg_rgb" and "bg_rgb" fields.
     _DTYPE_RGB = np.dtype(
         {
-            "names": ["ch", "fg", "bg", "fg_rgb", "bg_rgb"],
-            "formats": [np.int32, "(4,)u1", "(4,)u1", "(3,)u1", "(3,)u1"],
-            "offsets": [0, 4, 8, 4, 8],
+            "names": ["ch", "fg", "bg"],
+            "formats": [np.int32, "(3,)u1", "(3,)u1"],
+            "offsets": [0, 4, 8],
+            "itemsize": 12,
         }
     )
 
@@ -113,7 +114,7 @@ class Console:
         width: int,
         height: int,
         order: str = "C",
-        buffer: Optional[np.array] = None,
+        buffer: Optional[np.ndarray] = None,
     ):
         self._key_color = None  # type: Optional[Tuple[int, int, int]]
         self._order = tcod._internal.verify_order(order)
@@ -136,6 +137,7 @@ class Console:
             {
                 "w": width,
                 "h": height,
+                "elements": width * height,
                 "tiles": ffi.cast(
                     "struct TCOD_ConsoleTile*", self._tiles.ctypes.data
                 ),
@@ -206,7 +208,7 @@ class Console:
         return lib.TCOD_console_get_height(self.console_c)  # type: ignore
 
     @property
-    def bg(self) -> np.array:
+    def bg(self) -> np.ndarray:
         """A uint8 array with the shape (height, width, 3).
 
         You can change the consoles background colors by using this array.
@@ -221,7 +223,7 @@ class Console:
         return bg
 
     @property
-    def fg(self) -> np.array:
+    def fg(self) -> np.ndarray:
         """A uint8 array with the shape (height, width, 3).
 
         You can change the consoles foreground colors by using this array.
@@ -235,7 +237,7 @@ class Console:
         return fg
 
     @property
-    def ch(self) -> np.array:
+    def ch(self) -> np.ndarray:
         """An integer array with the shape (height, width).
 
         You can change the consoles character codes by using this array.
@@ -246,15 +248,15 @@ class Console:
         return self._tiles["ch"].T if self._order == "F" else self._tiles["ch"]
 
     @property
-    def tiles(self) -> np.array:
-        """An array of this consoles tile data.
+    def tiles(self) -> np.ndarray:
+        """An array of this consoles raw tile data.
 
         This acts as a combination of the `ch`, `fg`, and `bg` attributes.
         Colors include an alpha channel but how alpha works is currently
         undefined.
 
-        Example::
-            >>> con = tcod.console.Console(10, 2, order="F")
+        Example:
+            >>> con = tcod.console.Console(10, 2)
             >>> con.tiles[0, 0] = (
             ...     ord("X"),
             ...     (*tcod.white, 255),
@@ -267,6 +269,50 @@ class Console:
         """
         return self._tiles.T if self._order == "F" else self._tiles
 
+    @property  # type: ignore
+    @deprecate("This attribute has been renamed to `tiles`.")
+    def buffer(self) -> np.ndarray:
+        """An array of this consoles raw tile data.
+
+        .. versionadded:: 11.4
+
+        .. deprecated:: 11.8
+            Use :any:`Console.tiles` instead.
+        """
+        return self._tiles.T if self._order == "F" else self._tiles
+
+    @property
+    def tiles_rgb(self) -> np.ndarray:
+        """An array of this consoles tile data without the alpha channel.
+
+        The dtype of this array is effectively:
+        ``[("ch", np.intc), ("fg", "(3,)u1"), ("bg", "(3,)u1")]``
+
+        Example:
+            >>> con = tcod.console.Console(10, 2)
+            >>> con.tiles_rgb[0, 0] = ord("@"), tcod.yellow, tcod.black
+            >>> con.tiles_rgb[0, 0]
+            (64, [255, 255,   0], [0, 0, 0])
+            >>> con.tiles_rgb["bg"] = tcod.blue
+            >>> con.tiles_rgb[0, 0]
+            (64, [255, 255,   0], [  0,   0, 255])
+
+        .. versionadded:: 11.8
+        """
+        return self.tiles.view(self._DTYPE_RGB)
+
+    @property  # type: ignore
+    @deprecate("This attribute has been renamed to `tiles_rgb`.")
+    def tiles2(self) -> np.ndarray:
+        """This name is deprecated in favour of :any:`tiles_rgb`.
+
+        .. versionadded:: 11.3
+
+        .. deprecated:: 11.8
+            Use :any:`Console.tiles_rgb` instead.
+        """
+        return self.tiles_rgb
+
     @property
     def default_bg(self) -> Tuple[int, int, int]:
         """Tuple[int, int, int]: The default background color."""
@@ -318,11 +364,11 @@ class Console:
             stacklevel=3,
         )
 
-    def clear(  # type: ignore
+    def clear(
         self,
         ch: int = ord(" "),
-        fg: Tuple[int, int, int] = ...,
-        bg: Tuple[int, int, int] = ...,
+        fg: Tuple[int, int, int] = ...,  # type: ignore
+        bg: Tuple[int, int, int] = ...,  # type: ignore
     ) -> None:
         """Reset all values in this console to a single value.
 
@@ -543,8 +589,8 @@ class Console:
         """
         string_ = string.encode("utf-8")
         return int(
-            lib.get_height_rect(
-                self.console_c, x, y, width, height, string_, len(string_)
+            lib.TCOD_console_get_height_rect_n(
+                self.console_c, x, y, width, height, len(string_), string_
             )
         )
 
@@ -696,9 +742,9 @@ class Console:
         """Blit from this console onto the ``dest`` console.
 
         Args:
-            dest (Console): The destintaion console to blit onto.
-            dest_x (int): Leftmost coordinate of the destintaion console.
-            dest_y (int): Topmost coordinate of the destintaion console.
+            dest (Console): The destination console to blit onto.
+            dest_x (int): Leftmost coordinate of the destination console.
+            dest_y (int): Topmost coordinate of the destination console.
             src_x (int): X coordinate from this console to blit, from the left.
             src_y (int): Y coordinate from this console to blit, from the top.
             width (int): The width of the region to blit.
@@ -717,18 +763,23 @@ class Console:
 
             Previously they were:
             `(x, y, width, height, dest, dest_x, dest_y, *)`
+
+        .. versionchanged:: 11.6
+            Now supports per-cell alpha transparency.
+
+            Use :any:`Console.buffer` to set tile alpha before blit.
         """
         # The old syntax is easy to detect and correct.
         if hasattr(src_y, "console_c"):
-            (
-                src_x,  # type: ignore
-                src_y,
-                width,
-                height,
+            (src_x, src_y, width, height, dest, dest_x, dest_y,) = (
                 dest,  # type: ignore
                 dest_x,
                 dest_y,
-            ) = (dest, dest_x, dest_y, src_x, src_y, width, height)
+                src_x,
+                src_y,  # type: ignore
+                width,
+                height,
+            )
             warnings.warn(
                 "Parameter names have been moved around, see documentation.",
                 DeprecationWarning,
@@ -788,22 +839,39 @@ class Console:
 
         This is useful for some Python IDE's like IDLE, where the window would
         not be closed on its own otherwise.
+
+        .. seealso::
+            :any:`tcod.console_init_root`
         """
         if self.console_c != ffi.NULL:
             raise NotImplementedError("Only the root console has a context.")
         return self
 
+    def close(self) -> None:
+        """Close the active window managed by libtcod.
+
+        This must only be called on the root console, which is returned from
+        :any:`tcod.console_init_root`.
+
+        .. versionadded:: 11.11
+        """
+        if self.console_c != ffi.NULL:
+            raise NotImplementedError(
+                "Only the root console can be used to close libtcod's window."
+            )
+        lib.TCOD_console_delete(self.console_c)
+
     def __exit__(self, *args: Any) -> None:
         """Closes the graphical window on exit.
 
-        Some tcod functions may have undefined behaviour after this point.
+        Some tcod functions may have undefined behavior after this point.
         """
-        lib.TCOD_console_delete(self.console_c)
+        self.close()
 
     def __bool__(self) -> bool:
         """Returns False if this is the root console.
 
-        This mimics libtcodpy behaviour.
+        This mimics libtcodpy behavior.
         """
         return bool(self.console_c != ffi.NULL)
 
@@ -849,7 +917,7 @@ class Console:
         return (
             "tcod.console.Console(width=%i, height=%i, "
             "order=%r,buffer=\n%r)"
-            % (self.width, self.height, self._order, self.tiles)
+            % (self.width, self.height, self._order, self.buffer)
         )
 
     def __str__(self) -> str:
@@ -908,12 +976,12 @@ class Console:
         """
         x, y = self._pythonic_index(x, y)
         string_ = string.encode("utf-8")  # type: bytes
-        lib.console_print(
+        lib.TCOD_console_printn(
             self.console_c,
             x,
             y,
-            string_,
             len(string_),
+            string_,
             (fg,) if fg is not None else ffi.NULL,
             (bg,) if bg is not None else ffi.NULL,
             bg_blend,
@@ -940,7 +1008,7 @@ class Console:
         may change in future versions.
 
         `width` and `height` determine the bounds of the rectangle, the text
-        will automatically be broken to fit within these bounds.
+        will automatically be word-wrapped to fit within these bounds.
 
         `string` is a Unicode string which may include color control
         characters.
@@ -964,14 +1032,14 @@ class Console:
         x, y = self._pythonic_index(x, y)
         string_ = string.encode("utf-8")  # type: bytes
         return int(
-            lib.print_rect(
+            lib.TCOD_console_printn_rect(
                 self.console_c,
                 x,
                 y,
                 width,
                 height,
-                string_,
                 len(string_),
+                string_,
                 (fg,) if fg is not None else ffi.NULL,
                 (bg,) if bg is not None else ffi.NULL,
                 bg_blend,
@@ -1018,14 +1086,14 @@ class Console:
         """
         x, y = self._pythonic_index(x, y)
         title_ = title.encode("utf-8")  # type: bytes
-        lib.print_frame(
+        lib.TCOD_console_printn_frame(
             self.console_c,
             x,
             y,
             width,
             height,
-            title_,
             len(title_),
+            title_,
             (fg,) if fg is not None else ffi.NULL,
             (bg,) if bg is not None else ffi.NULL,
             bg_blend,
@@ -1068,7 +1136,7 @@ class Console:
             `fg` and `bg` now default to `None` instead of white-on-black.
         """
         x, y = self._pythonic_index(x, y)
-        lib.draw_rect(
+        lib.TCOD_console_draw_rect_rgb(
             self.console_c,
             x,
             y,
@@ -1080,6 +1148,22 @@ class Console:
             bg_blend,
         )
 
+    def draw_semigraphics(self, pixels: Any, x: int = 0, y: int = 0) -> None:
+        """Draw a block of 2x2 semi-graphics into this console.
+
+        `pixels` is an Image or an array-like object.  It will be down-sampled
+        into 2x2 blocks when drawn.  Array-like objects must be in the shape of
+        `(height, width, RGB)` and should have a `dtype` of `numpy.uint8`.
+
+        `x` and `y` is the upper-left tile position to start drawing.
+
+        .. versionadded:: 11.4
+        """
+        image = tcod._internal._asimage(pixels)
+        lib.TCOD_image_blit_2x(
+            image.image_c, self.console_c, x, y, 0, 0, -1, -1
+        )
+
 
 def get_height_rect(width: int, string: str) -> int:
     """Return the number of lines which would be printed from these parameters.
@@ -1091,4 +1175,34 @@ def get_height_rect(width: int, string: str) -> int:
     .. versionadded:: 9.2
     """
     string_ = string.encode("utf-8")  # type: bytes
-    return int(lib.get_height_rect2(width, string_, len(string_)))
+    return int(
+        lib.TCOD_console_get_height_rect_wn(width, len(string_), string_)
+    )
+
+
+def recommended_size() -> Tuple[int, int]:
+    """Return the recommended size of a console for the current active window.
+
+    The return is determined from the active tileset size and active window
+    size.  This result should be used create an :any:`Console` instance.
+
+    This function will raise RuntimeError if libtcod has not been initialized.
+
+    .. versionadded:: 11.8
+
+    .. seealso::
+        :any:`tcod.console_init_root`
+        :any:`tcod.console_flush`
+    """
+    if not lib.TCOD_ctx.engine:
+        raise RuntimeError("The libtcod engine was not initialized first.")
+    window = lib.TCOD_sys_get_sdl_window()
+    renderer = lib.TCOD_sys_get_sdl_renderer()
+    with ffi.new("int[2]") as xy:
+        if renderer:
+            lib.SDL_GetRendererOutputSize(renderer, xy, xy + 1)
+        else:  # Assume OpenGL if a renderer does not exist.
+            lib.SDL_GL_GetDrawableSize(window, xy, xy + 1)
+        w = max(1, xy[0] // lib.TCOD_ctx.tileset.tile_width)
+        h = max(1, xy[1] // lib.TCOD_ctx.tileset.tile_height)
+    return w, h
