@@ -1,6 +1,11 @@
 """
-An alternative, more direct implementation of event handling based on using
-cffi calls to SDL functions.  The current code is partially incomplete.
+A light-weight implementation of event handling built on calls to SDL.
+
+Many event constants are derived directly from SDL.
+For example: ``tcod.event.K_UP`` and ``tcod.event.SCANCODE_A`` refer to
+SDL's ``SDLK_UP`` and ``SDL_SCANCODE_A`` respectfully.
+`See this table for all of SDL's keyboard constants.
+<https://wiki.libsdl.org/SDL_Keycode>`_
 
 Printing any event will tell you its attributes in a human readable format.
 An events type attribute if omitted is just the classes name with all letters
@@ -15,26 +20,40 @@ implied by ``import tcod``.
 
 .. versionadded:: 8.4
 """
-from typing import Any, Dict, NamedTuple, Optional, Iterator, Tuple
+from typing import Any, Dict, Mapping, NamedTuple, Optional, Iterator, Tuple
 
-import tcod
 import tcod.event_constants
+from tcod.loader import ffi, lib
 from tcod.event_constants import *  # noqa: F4
+from tcod.event_constants import KMOD_SHIFT, KMOD_CTRL, KMOD_ALT, KMOD_GUI
+
+
+class _ConstantsWithPrefix(Mapping[int, str]):
+    def __init__(self, constants: Mapping[int, str]):
+        self.constants = constants
+
+    def __getitem__(self, key: int) -> str:
+        return "tcod.event." + self.constants[key]
+
+    def __len__(self) -> int:
+        return len(self.constants)
+
+    def __iter__(self) -> Iterator[int]:
+        return iter(self.constants)
 
 
 def _describe_bitmask(
-    bits: int, table: Dict[Any, str], default: str = "0"
+    bits: int, table: Mapping[int, str], default: str = "0"
 ) -> str:
-    """Returns a bitmask in human readable form.
+    """Return a bitmask in human readable form.
 
     This is a private function, used internally.
 
-    Args:
-        bits (int): The bitmask to be represented.
-        table (Dict[Any,str]): A reverse lookup table.
-        default (Any): A default return value when bits is 0.
+    `bits` is the bitmask to be represented.
 
-    Returns: str: A printable version of the bits variable.
+    `table` is a reverse lookup table.
+
+    `default` is returned when no other bits can be represented.
     """
     result = []
     for bit, name in table.items():
@@ -47,49 +66,62 @@ def _describe_bitmask(
 
 def _pixel_to_tile(x: float, y: float) -> Tuple[float, float]:
     """Convert pixel coordinates to tile coordinates."""
-    xy = tcod.ffi.new("double[2]", (x, y))
-    tcod.lib.TCOD_sys_pixel_to_tile(xy, xy + 1)
+    xy = ffi.new("double[2]", (x, y))
+    lib.TCOD_sys_pixel_to_tile(xy, xy + 1)
     return xy[0], xy[1]
 
 
 Point = NamedTuple("Point", [("x", float), ("y", float)])
 
 # manually define names for SDL macros
-BUTTON_LEFT = 1
-BUTTON_MIDDLE = 2
-BUTTON_RIGHT = 3
-BUTTON_X1 = 4
-BUTTON_X2 = 5
-BUTTON_LMASK = 0x01
-BUTTON_MMASK = 0x02
-BUTTON_RMASK = 0x04
-BUTTON_X1MASK = 0x08
-BUTTON_X2MASK = 0x10
-
-KMOD_SHIFT = (
-    tcod.event_constants.KMOD_LSHIFT | tcod.event_constants.KMOD_RSHIFT
-)
-KMOD_CTRL = tcod.event_constants.KMOD_LCTRL | tcod.event_constants.KMOD_RCTRL
-KMOD_ALT = tcod.event_constants.KMOD_LALT | tcod.event_constants.KMOD_RALT
-KMOD_GUI = tcod.event_constants.KMOD_LGUI | tcod.event_constants.KMOD_RGUI
+BUTTON_LEFT = lib.SDL_BUTTON_LEFT
+BUTTON_MIDDLE = lib.SDL_BUTTON_MIDDLE
+BUTTON_RIGHT = lib.SDL_BUTTON_RIGHT
+BUTTON_X1 = lib.SDL_BUTTON_X1
+BUTTON_X2 = lib.SDL_BUTTON_X2
+BUTTON_LMASK = lib.SDL_BUTTON_LMASK
+BUTTON_MMASK = lib.SDL_BUTTON_MMASK
+BUTTON_RMASK = lib.SDL_BUTTON_RMASK
+BUTTON_X1MASK = lib.SDL_BUTTON_X1MASK
+BUTTON_X2MASK = lib.SDL_BUTTON_X2MASK
 
 # reverse tables are used to get the tcod.event name from the value.
 _REVERSE_BUTTON_TABLE = {
-    BUTTON_LEFT: "tcod.event.BUTTON_LEFT",
-    BUTTON_MIDDLE: "tcod.event.BUTTON_MIDDLE",
-    BUTTON_RIGHT: "tcod.event.BUTTON_RIGHT",
-    BUTTON_X1: "tcod.event.BUTTON_X1",
-    BUTTON_X2: "tcod.event.BUTTON_X2",
+    BUTTON_LEFT: "BUTTON_LEFT",
+    BUTTON_MIDDLE: "BUTTON_MIDDLE",
+    BUTTON_RIGHT: "BUTTON_RIGHT",
+    BUTTON_X1: "BUTTON_X1",
+    BUTTON_X2: "BUTTON_X2",
 }
 
 _REVERSE_BUTTON_MASK_TABLE = {
-    BUTTON_LMASK: "tcod.event.BUTTON_LMASK",
-    BUTTON_MMASK: "tcod.event.BUTTON_MMASK",
-    BUTTON_RMASK: "tcod.event.BUTTON_RMASK",
-    BUTTON_X1MASK: "tcod.event.BUTTON_X1MASK",
-    BUTTON_X2MASK: "tcod.event.BUTTON_X2MASK",
+    BUTTON_LMASK: "BUTTON_LMASK",
+    BUTTON_MMASK: "BUTTON_MMASK",
+    BUTTON_RMASK: "BUTTON_RMASK",
+    BUTTON_X1MASK: "BUTTON_X1MASK",
+    BUTTON_X2MASK: "BUTTON_X2MASK",
 }
 
+_REVERSE_BUTTON_TABLE_PREFIX = _ConstantsWithPrefix(_REVERSE_BUTTON_TABLE)
+_REVERSE_BUTTON_MASK_TABLE_PREFIX = _ConstantsWithPrefix(
+    _REVERSE_BUTTON_MASK_TABLE
+)
+_REVERSE_SCANCODE_TABLE_PREFIX = _ConstantsWithPrefix(
+    tcod.event_constants._REVERSE_SCANCODE_TABLE
+)
+_REVERSE_SYM_TABLE_PREFIX = _ConstantsWithPrefix(
+    tcod.event_constants._REVERSE_SYM_TABLE
+)
+
+
+_REVERSE_MOD_TABLE = tcod.event_constants._REVERSE_MOD_TABLE.copy()
+del _REVERSE_MOD_TABLE[KMOD_SHIFT]
+del _REVERSE_MOD_TABLE[KMOD_CTRL]
+del _REVERSE_MOD_TABLE[KMOD_ALT]
+del _REVERSE_MOD_TABLE[KMOD_GUI]
+
+_REVERSE_MOD_TABLE_PREFIX = _ConstantsWithPrefix(_REVERSE_MOD_TABLE)
+
 
 class Event:
     """The base event class.
@@ -111,6 +143,9 @@ class Event:
         """Return a class instance from a python-cffi 'SDL_Event*' pointer."""
         raise NotImplementedError()
 
+    def __str__(self) -> str:
+        return "<type=%r>" % (self.type,)
+
 
 class Quit(Event):
     """An application quit request event.
@@ -129,7 +164,7 @@ class Quit(Event):
         return self
 
     def __repr__(self) -> str:
-        return "tcod.event.%s()" % self.__class__.__name__
+        return "tcod.event.%s()" % (self.__class__.__name__,)
 
 
 class KeyboardEvent(Event):
@@ -199,17 +234,40 @@ class KeyboardEvent(Event):
         self.sdl_event = sdl_event
         return self
 
+    def _scancode_constant(self, table: Mapping[int, str]) -> str:
+        """Return the constant name for this scan-code from a table."""
+        try:
+            return table[self.scancode]
+        except KeyError:
+            return str(self.scancode)
+
+    def _sym_constant(self, table: Mapping[int, str]) -> str:
+        """Return the constant name for this symbol from a table."""
+        try:
+            return table[self.sym]
+        except KeyError:
+            return str(self.sym)
+
     def __repr__(self) -> str:
         return "tcod.event.%s(scancode=%s, sym=%s, mod=%s%s)" % (
             self.__class__.__name__,
-            tcod.event_constants._REVERSE_SCANCODE_TABLE[self.scancode],
-            tcod.event_constants._REVERSE_SYM_TABLE[self.sym],
-            _describe_bitmask(
-                self.mod, tcod.event_constants._REVERSE_MOD_TABLE
-            ),
+            self._scancode_constant(_REVERSE_SCANCODE_TABLE_PREFIX),
+            self._sym_constant(_REVERSE_SYM_TABLE_PREFIX),
+            _describe_bitmask(self.mod, _REVERSE_MOD_TABLE_PREFIX),
             ", repeat=True" if self.repeat else "",
         )
 
+    def __str__(self) -> str:
+        return "<%s, scancode=%s, sym=%s, mod=%s, repeat=%r>" % (
+            super().__str__().strip("<>"),
+            self._scancode_constant(
+                tcod.event_constants._REVERSE_SCANCODE_TABLE
+            ),
+            self._sym_constant(tcod.event_constants._REVERSE_SYM_TABLE),
+            _describe_bitmask(self.mod, _REVERSE_MOD_TABLE),
+            self.repeat,
+        )
+
 
 class KeyDown(KeyboardEvent):
     pass
@@ -235,7 +293,7 @@ class MouseState(Event):
             * tcod.event.BUTTON_X1MASK
             * tcod.event.BUTTON_X2MASK
 
-    .. addedversion:: 9.3
+    .. versionadded:: 9.3
     """
 
     def __init__(
@@ -252,8 +310,16 @@ class MouseState(Event):
     def __repr__(self) -> str:
         return ("tcod.event.%s(pixel=%r, tile=%r, state=%s)") % (
             self.__class__.__name__,
-            self.pixel,
-            self.tile,
+            tuple(self.pixel),
+            tuple(self.tile),
+            _describe_bitmask(self.state, _REVERSE_BUTTON_MASK_TABLE_PREFIX),
+        )
+
+    def __str__(self) -> str:
+        return ("<%s, pixel=(x=%i, y=%i), tile=(x=%i, y=%i), state=%s>") % (
+            super().__str__().strip("<>"),
+            *self.pixel,
+            *self.tile,
             _describe_bitmask(self.state, _REVERSE_BUTTON_MASK_TABLE),
         )
 
@@ -311,11 +377,20 @@ class MouseMotion(MouseState):
             "tile=%r, tile_motion=%r, state=%s)"
         ) % (
             self.__class__.__name__,
-            self.pixel,
-            self.pixel_motion,
-            self.tile,
-            self.tile_motion,
-            _describe_bitmask(self.state, _REVERSE_BUTTON_MASK_TABLE),
+            tuple(self.pixel),
+            tuple(self.pixel_motion),
+            tuple(self.tile),
+            tuple(self.tile_motion),
+            _describe_bitmask(self.state, _REVERSE_BUTTON_MASK_TABLE_PREFIX),
+        )
+
+    def __str__(self) -> str:
+        return (
+            "<%s, pixel_motion=(x=%i, y=%i), tile_motion=(x=%i, y=%i)>"
+        ) % (
+            super().__str__().strip("<>"),
+            *self.pixel_motion,
+            *self.tile_motion,
         )
 
 
@@ -366,9 +441,20 @@ class MouseButtonEvent(MouseState):
     def __repr__(self) -> str:
         return "tcod.event.%s(pixel=%r, tile=%r, button=%s)" % (
             self.__class__.__name__,
-            self.pixel,
-            self.tile,
-            _REVERSE_BUTTON_TABLE[self.button],
+            tuple(self.pixel),
+            tuple(self.tile),
+            _REVERSE_BUTTON_TABLE_PREFIX[self.button],
+        )
+
+    def __str__(self) -> str:
+        return (
+            "<type=%r, pixel=(x=%i, y=%i), tile=(x=%i, y=%i), button=%s)"
+            % (
+                self.type,
+                *self.pixel,
+                *self.tile,
+                _REVERSE_BUTTON_TABLE[self.button],
+            )
         )
 
 
@@ -413,6 +499,14 @@ class MouseWheel(Event):
             ", flipped=True" if self.flipped else "",
         )
 
+    def __str__(self) -> str:
+        return "<%s, x=%i, y=%i, flipped=%r)" % (
+            super().__str__().strip("<>"),
+            self.x,
+            self.y,
+            self.flipped,
+        )
+
 
 class TextInput(Event):
     """
@@ -427,13 +521,16 @@ class TextInput(Event):
 
     @classmethod
     def from_sdl_event(cls, sdl_event: Any) -> "TextInput":
-        self = cls(tcod.ffi.string(sdl_event.text.text, 32).decode("utf8"))
+        self = cls(ffi.string(sdl_event.text.text, 32).decode("utf8"))
         self.sdl_event = sdl_event
         return self
 
     def __repr__(self) -> str:
         return "tcod.event.%s(text=%r)" % (self.__class__.__name__, self.text)
 
+    def __str__(self) -> str:
+        return "<%s, text=%r)" % (super().__str__().strip("<>"), self.text)
+
 
 class WindowEvent(Event):
     """
@@ -447,11 +544,11 @@ class WindowEvent(Event):
             return Undefined.from_sdl_event(sdl_event)
         event_type = cls.__WINDOW_TYPES[sdl_event.window.event].upper()
         self = None  # type: Any
-        if sdl_event.window.event == tcod.lib.SDL_WINDOWEVENT_MOVED:
+        if sdl_event.window.event == lib.SDL_WINDOWEVENT_MOVED:
             self = WindowMoved(sdl_event.window.data1, sdl_event.window.data2)
         elif sdl_event.window.event in (
-            tcod.lib.SDL_WINDOWEVENT_RESIZED,
-            tcod.lib.SDL_WINDOWEVENT_SIZE_CHANGED,
+            lib.SDL_WINDOWEVENT_RESIZED,
+            lib.SDL_WINDOWEVENT_SIZE_CHANGED,
         ):
             self = WindowResized(
                 event_type, sdl_event.window.data1, sdl_event.window.data2
@@ -465,22 +562,22 @@ class WindowEvent(Event):
         return "tcod.event.%s(type=%r)" % (self.__class__.__name__, self.type)
 
     __WINDOW_TYPES = {
-        tcod.lib.SDL_WINDOWEVENT_SHOWN: "WindowShown",
-        tcod.lib.SDL_WINDOWEVENT_HIDDEN: "WindowHidden",
-        tcod.lib.SDL_WINDOWEVENT_EXPOSED: "WindowExposed",
-        tcod.lib.SDL_WINDOWEVENT_MOVED: "WindowMoved",
-        tcod.lib.SDL_WINDOWEVENT_RESIZED: "WindowResized",
-        tcod.lib.SDL_WINDOWEVENT_SIZE_CHANGED: "WindowSizeChanged",
-        tcod.lib.SDL_WINDOWEVENT_MINIMIZED: "WindowMinimized",
-        tcod.lib.SDL_WINDOWEVENT_MAXIMIZED: "WindowMaximized",
-        tcod.lib.SDL_WINDOWEVENT_RESTORED: "WindowRestored",
-        tcod.lib.SDL_WINDOWEVENT_ENTER: "WindowEnter",
-        tcod.lib.SDL_WINDOWEVENT_LEAVE: "WindowLeave",
-        tcod.lib.SDL_WINDOWEVENT_FOCUS_GAINED: "WindowFocusGained",
-        tcod.lib.SDL_WINDOWEVENT_FOCUS_LOST: "WindowFocusLost",
-        tcod.lib.SDL_WINDOWEVENT_CLOSE: "WindowClose",
-        tcod.lib.SDL_WINDOWEVENT_TAKE_FOCUS: "WindowTakeFocus",
-        tcod.lib.SDL_WINDOWEVENT_HIT_TEST: "WindowHitTest",
+        lib.SDL_WINDOWEVENT_SHOWN: "WindowShown",
+        lib.SDL_WINDOWEVENT_HIDDEN: "WindowHidden",
+        lib.SDL_WINDOWEVENT_EXPOSED: "WindowExposed",
+        lib.SDL_WINDOWEVENT_MOVED: "WindowMoved",
+        lib.SDL_WINDOWEVENT_RESIZED: "WindowResized",
+        lib.SDL_WINDOWEVENT_SIZE_CHANGED: "WindowSizeChanged",
+        lib.SDL_WINDOWEVENT_MINIMIZED: "WindowMinimized",
+        lib.SDL_WINDOWEVENT_MAXIMIZED: "WindowMaximized",
+        lib.SDL_WINDOWEVENT_RESTORED: "WindowRestored",
+        lib.SDL_WINDOWEVENT_ENTER: "WindowEnter",
+        lib.SDL_WINDOWEVENT_LEAVE: "WindowLeave",
+        lib.SDL_WINDOWEVENT_FOCUS_GAINED: "WindowFocusGained",
+        lib.SDL_WINDOWEVENT_FOCUS_LOST: "WindowFocusLost",
+        lib.SDL_WINDOWEVENT_CLOSE: "WindowClose",
+        lib.SDL_WINDOWEVENT_TAKE_FOCUS: "WindowTakeFocus",
+        lib.SDL_WINDOWEVENT_HIT_TEST: "WindowHitTest",
     }
 
 
@@ -505,6 +602,13 @@ class WindowMoved(WindowEvent):
             self.y,
         )
 
+    def __str__(self) -> str:
+        return "<%s, x=%r, y=%r)" % (
+            super().__str__().strip("<>"),
+            self.x,
+            self.y,
+        )
+
 
 class WindowResized(WindowEvent):
     """
@@ -527,6 +631,13 @@ class WindowResized(WindowEvent):
             self.height,
         )
 
+    def __str__(self) -> str:
+        return "<%s, width=%r, height=%r)" % (
+            super().__str__().strip("<>"),
+            self.width,
+            self.height,
+        )
+
 
 class Undefined(Event):
     """This class is a place holder for SDL events without their own tcod.event
@@ -549,15 +660,15 @@ class Undefined(Event):
 
 
 _SDL_TO_CLASS_TABLE = {
-    tcod.lib.SDL_QUIT: Quit,
-    tcod.lib.SDL_KEYDOWN: KeyDown,
-    tcod.lib.SDL_KEYUP: KeyUp,
-    tcod.lib.SDL_MOUSEMOTION: MouseMotion,
-    tcod.lib.SDL_MOUSEBUTTONDOWN: MouseButtonDown,
-    tcod.lib.SDL_MOUSEBUTTONUP: MouseButtonUp,
-    tcod.lib.SDL_MOUSEWHEEL: MouseWheel,
-    tcod.lib.SDL_TEXTINPUT: TextInput,
-    tcod.lib.SDL_WINDOWEVENT: WindowEvent,
+    lib.SDL_QUIT: Quit,
+    lib.SDL_KEYDOWN: KeyDown,
+    lib.SDL_KEYUP: KeyUp,
+    lib.SDL_MOUSEMOTION: MouseMotion,
+    lib.SDL_MOUSEBUTTONDOWN: MouseButtonDown,
+    lib.SDL_MOUSEBUTTONUP: MouseButtonUp,
+    lib.SDL_MOUSEWHEEL: MouseWheel,
+    lib.SDL_TEXTINPUT: TextInput,
+    lib.SDL_WINDOWEVENT: WindowEvent,
 }  # type: Dict[int, Any]
 
 
@@ -582,8 +693,8 @@ def get() -> Iterator[Any]:
             else:
                 print(event)
     """
-    sdl_event = tcod.ffi.new("SDL_Event*")
-    while tcod.lib.SDL_PollEvent(sdl_event):
+    sdl_event = ffi.new("SDL_Event*")
+    while lib.SDL_PollEvent(sdl_event):
         if sdl_event.type in _SDL_TO_CLASS_TABLE:
             yield _SDL_TO_CLASS_TABLE[sdl_event.type].from_sdl_event(sdl_event)
         else:
@@ -614,9 +725,9 @@ def wait(timeout: Optional[float] = None) -> Iterator[Any]:
                 print(event)
     """
     if timeout is not None:
-        tcod.lib.SDL_WaitEventTimeout(tcod.ffi.NULL, int(timeout * 1000))
+        lib.SDL_WaitEventTimeout(ffi.NULL, int(timeout * 1000))
     else:
-        tcod.lib.SDL_WaitEvent(tcod.ffi.NULL)
+        lib.SDL_WaitEvent(ffi.NULL)
     return get()
 
 
@@ -749,15 +860,20 @@ class EventDispatch:
 def get_mouse_state() -> MouseState:
     """Return the current state of the mouse.
 
-    .. addedversion:: 9.3
+    .. versionadded:: 9.3
     """
-    xy = tcod.ffi.new("int[2]")
-    buttons = tcod.lib.SDL_GetMouseState(xy, xy + 1)
+    xy = ffi.new("int[2]")
+    buttons = lib.SDL_GetMouseState(xy, xy + 1)
     x, y = _pixel_to_tile(*xy)
     return MouseState((xy[0], xy[1]), (int(x), int(y)), buttons)
 
 
-__all__ = [
+@ffi.def_extern()  # type: ignore
+def _pycall_event_watch(userdata: Any, sdl_event: Any) -> int:
+    return 0
+
+
+__all__ = [  # noqa: F405
     "Point",
     "BUTTON_LEFT",
     "BUTTON_MIDDLE",
@@ -786,5 +902,505 @@ __all__ = [
     "Undefined",
     "get",
     "wait",
+    "get_mouse_state",
     "EventDispatch",
-] + tcod.event_constants.__all__
+    # --- From event_constants.py ---
+    "SCANCODE_UNKNOWN",
+    "SCANCODE_A",
+    "SCANCODE_B",
+    "SCANCODE_C",
+    "SCANCODE_D",
+    "SCANCODE_E",
+    "SCANCODE_F",
+    "SCANCODE_G",
+    "SCANCODE_H",
+    "SCANCODE_I",
+    "SCANCODE_J",
+    "SCANCODE_K",
+    "SCANCODE_L",
+    "SCANCODE_M",
+    "SCANCODE_N",
+    "SCANCODE_O",
+    "SCANCODE_P",
+    "SCANCODE_Q",
+    "SCANCODE_R",
+    "SCANCODE_S",
+    "SCANCODE_T",
+    "SCANCODE_U",
+    "SCANCODE_V",
+    "SCANCODE_W",
+    "SCANCODE_X",
+    "SCANCODE_Y",
+    "SCANCODE_Z",
+    "SCANCODE_1",
+    "SCANCODE_2",
+    "SCANCODE_3",
+    "SCANCODE_4",
+    "SCANCODE_5",
+    "SCANCODE_6",
+    "SCANCODE_7",
+    "SCANCODE_8",
+    "SCANCODE_9",
+    "SCANCODE_0",
+    "SCANCODE_RETURN",
+    "SCANCODE_ESCAPE",
+    "SCANCODE_BACKSPACE",
+    "SCANCODE_TAB",
+    "SCANCODE_SPACE",
+    "SCANCODE_MINUS",
+    "SCANCODE_EQUALS",
+    "SCANCODE_LEFTBRACKET",
+    "SCANCODE_RIGHTBRACKET",
+    "SCANCODE_BACKSLASH",
+    "SCANCODE_NONUSHASH",
+    "SCANCODE_SEMICOLON",
+    "SCANCODE_APOSTROPHE",
+    "SCANCODE_GRAVE",
+    "SCANCODE_COMMA",
+    "SCANCODE_PERIOD",
+    "SCANCODE_SLASH",
+    "SCANCODE_CAPSLOCK",
+    "SCANCODE_F1",
+    "SCANCODE_F2",
+    "SCANCODE_F3",
+    "SCANCODE_F4",
+    "SCANCODE_F5",
+    "SCANCODE_F6",
+    "SCANCODE_F7",
+    "SCANCODE_F8",
+    "SCANCODE_F9",
+    "SCANCODE_F10",
+    "SCANCODE_F11",
+    "SCANCODE_F12",
+    "SCANCODE_PRINTSCREEN",
+    "SCANCODE_SCROLLLOCK",
+    "SCANCODE_PAUSE",
+    "SCANCODE_INSERT",
+    "SCANCODE_HOME",
+    "SCANCODE_PAGEUP",
+    "SCANCODE_DELETE",
+    "SCANCODE_END",
+    "SCANCODE_PAGEDOWN",
+    "SCANCODE_RIGHT",
+    "SCANCODE_LEFT",
+    "SCANCODE_DOWN",
+    "SCANCODE_UP",
+    "SCANCODE_NUMLOCKCLEAR",
+    "SCANCODE_KP_DIVIDE",
+    "SCANCODE_KP_MULTIPLY",
+    "SCANCODE_KP_MINUS",
+    "SCANCODE_KP_PLUS",
+    "SCANCODE_KP_ENTER",
+    "SCANCODE_KP_1",
+    "SCANCODE_KP_2",
+    "SCANCODE_KP_3",
+    "SCANCODE_KP_4",
+    "SCANCODE_KP_5",
+    "SCANCODE_KP_6",
+    "SCANCODE_KP_7",
+    "SCANCODE_KP_8",
+    "SCANCODE_KP_9",
+    "SCANCODE_KP_0",
+    "SCANCODE_KP_PERIOD",
+    "SCANCODE_NONUSBACKSLASH",
+    "SCANCODE_APPLICATION",
+    "SCANCODE_POWER",
+    "SCANCODE_KP_EQUALS",
+    "SCANCODE_F13",
+    "SCANCODE_F14",
+    "SCANCODE_F15",
+    "SCANCODE_F16",
+    "SCANCODE_F17",
+    "SCANCODE_F18",
+    "SCANCODE_F19",
+    "SCANCODE_F20",
+    "SCANCODE_F21",
+    "SCANCODE_F22",
+    "SCANCODE_F23",
+    "SCANCODE_F24",
+    "SCANCODE_EXECUTE",
+    "SCANCODE_HELP",
+    "SCANCODE_MENU",
+    "SCANCODE_SELECT",
+    "SCANCODE_STOP",
+    "SCANCODE_AGAIN",
+    "SCANCODE_UNDO",
+    "SCANCODE_CUT",
+    "SCANCODE_COPY",
+    "SCANCODE_PASTE",
+    "SCANCODE_FIND",
+    "SCANCODE_MUTE",
+    "SCANCODE_VOLUMEUP",
+    "SCANCODE_VOLUMEDOWN",
+    "SCANCODE_KP_COMMA",
+    "SCANCODE_KP_EQUALSAS400",
+    "SCANCODE_INTERNATIONAL1",
+    "SCANCODE_INTERNATIONAL2",
+    "SCANCODE_INTERNATIONAL3",
+    "SCANCODE_INTERNATIONAL4",
+    "SCANCODE_INTERNATIONAL5",
+    "SCANCODE_INTERNATIONAL6",
+    "SCANCODE_INTERNATIONAL7",
+    "SCANCODE_INTERNATIONAL8",
+    "SCANCODE_INTERNATIONAL9",
+    "SCANCODE_LANG1",
+    "SCANCODE_LANG2",
+    "SCANCODE_LANG3",
+    "SCANCODE_LANG4",
+    "SCANCODE_LANG5",
+    "SCANCODE_LANG6",
+    "SCANCODE_LANG7",
+    "SCANCODE_LANG8",
+    "SCANCODE_LANG9",
+    "SCANCODE_ALTERASE",
+    "SCANCODE_SYSREQ",
+    "SCANCODE_CANCEL",
+    "SCANCODE_CLEAR",
+    "SCANCODE_PRIOR",
+    "SCANCODE_RETURN2",
+    "SCANCODE_SEPARATOR",
+    "SCANCODE_OUT",
+    "SCANCODE_OPER",
+    "SCANCODE_CLEARAGAIN",
+    "SCANCODE_CRSEL",
+    "SCANCODE_EXSEL",
+    "SCANCODE_KP_00",
+    "SCANCODE_KP_000",
+    "SCANCODE_THOUSANDSSEPARATOR",
+    "SCANCODE_DECIMALSEPARATOR",
+    "SCANCODE_CURRENCYUNIT",
+    "SCANCODE_CURRENCYSUBUNIT",
+    "SCANCODE_KP_LEFTPAREN",
+    "SCANCODE_KP_RIGHTPAREN",
+    "SCANCODE_KP_LEFTBRACE",
+    "SCANCODE_KP_RIGHTBRACE",
+    "SCANCODE_KP_TAB",
+    "SCANCODE_KP_BACKSPACE",
+    "SCANCODE_KP_A",
+    "SCANCODE_KP_B",
+    "SCANCODE_KP_C",
+    "SCANCODE_KP_D",
+    "SCANCODE_KP_E",
+    "SCANCODE_KP_F",
+    "SCANCODE_KP_XOR",
+    "SCANCODE_KP_POWER",
+    "SCANCODE_KP_PERCENT",
+    "SCANCODE_KP_LESS",
+    "SCANCODE_KP_GREATER",
+    "SCANCODE_KP_AMPERSAND",
+    "SCANCODE_KP_DBLAMPERSAND",
+    "SCANCODE_KP_VERTICALBAR",
+    "SCANCODE_KP_DBLVERTICALBAR",
+    "SCANCODE_KP_COLON",
+    "SCANCODE_KP_HASH",
+    "SCANCODE_KP_SPACE",
+    "SCANCODE_KP_AT",
+    "SCANCODE_KP_EXCLAM",
+    "SCANCODE_KP_MEMSTORE",
+    "SCANCODE_KP_MEMRECALL",
+    "SCANCODE_KP_MEMCLEAR",
+    "SCANCODE_KP_MEMADD",
+    "SCANCODE_KP_MEMSUBTRACT",
+    "SCANCODE_KP_MEMMULTIPLY",
+    "SCANCODE_KP_MEMDIVIDE",
+    "SCANCODE_KP_PLUSMINUS",
+    "SCANCODE_KP_CLEAR",
+    "SCANCODE_KP_CLEARENTRY",
+    "SCANCODE_KP_BINARY",
+    "SCANCODE_KP_OCTAL",
+    "SCANCODE_KP_DECIMAL",
+    "SCANCODE_KP_HEXADECIMAL",
+    "SCANCODE_LCTRL",
+    "SCANCODE_LSHIFT",
+    "SCANCODE_LALT",
+    "SCANCODE_LGUI",
+    "SCANCODE_RCTRL",
+    "SCANCODE_RSHIFT",
+    "SCANCODE_RALT",
+    "SCANCODE_RGUI",
+    "SCANCODE_MODE",
+    "SCANCODE_AUDIONEXT",
+    "SCANCODE_AUDIOPREV",
+    "SCANCODE_AUDIOSTOP",
+    "SCANCODE_AUDIOPLAY",
+    "SCANCODE_AUDIOMUTE",
+    "SCANCODE_MEDIASELECT",
+    "SCANCODE_WWW",
+    "SCANCODE_MAIL",
+    "SCANCODE_CALCULATOR",
+    "SCANCODE_COMPUTER",
+    "SCANCODE_AC_SEARCH",
+    "SCANCODE_AC_HOME",
+    "SCANCODE_AC_BACK",
+    "SCANCODE_AC_FORWARD",
+    "SCANCODE_AC_STOP",
+    "SCANCODE_AC_REFRESH",
+    "SCANCODE_AC_BOOKMARKS",
+    "SCANCODE_BRIGHTNESSDOWN",
+    "SCANCODE_BRIGHTNESSUP",
+    "SCANCODE_DISPLAYSWITCH",
+    "SCANCODE_KBDILLUMTOGGLE",
+    "SCANCODE_KBDILLUMDOWN",
+    "SCANCODE_KBDILLUMUP",
+    "SCANCODE_EJECT",
+    "SCANCODE_SLEEP",
+    "SCANCODE_APP1",
+    "SCANCODE_APP2",
+    "K_UNKNOWN",
+    "K_BACKSPACE",
+    "K_TAB",
+    "K_RETURN",
+    "K_ESCAPE",
+    "K_SPACE",
+    "K_EXCLAIM",
+    "K_QUOTEDBL",
+    "K_HASH",
+    "K_DOLLAR",
+    "K_PERCENT",
+    "K_AMPERSAND",
+    "K_QUOTE",
+    "K_LEFTPAREN",
+    "K_RIGHTPAREN",
+    "K_ASTERISK",
+    "K_PLUS",
+    "K_COMMA",
+    "K_MINUS",
+    "K_PERIOD",
+    "K_SLASH",
+    "K_0",
+    "K_1",
+    "K_2",
+    "K_3",
+    "K_4",
+    "K_5",
+    "K_6",
+    "K_7",
+    "K_8",
+    "K_9",
+    "K_COLON",
+    "K_SEMICOLON",
+    "K_LESS",
+    "K_EQUALS",
+    "K_GREATER",
+    "K_QUESTION",
+    "K_AT",
+    "K_LEFTBRACKET",
+    "K_BACKSLASH",
+    "K_RIGHTBRACKET",
+    "K_CARET",
+    "K_UNDERSCORE",
+    "K_BACKQUOTE",
+    "K_a",
+    "K_b",
+    "K_c",
+    "K_d",
+    "K_e",
+    "K_f",
+    "K_g",
+    "K_h",
+    "K_i",
+    "K_j",
+    "K_k",
+    "K_l",
+    "K_m",
+    "K_n",
+    "K_o",
+    "K_p",
+    "K_q",
+    "K_r",
+    "K_s",
+    "K_t",
+    "K_u",
+    "K_v",
+    "K_w",
+    "K_x",
+    "K_y",
+    "K_z",
+    "K_DELETE",
+    "K_SCANCODE_MASK",
+    "K_CAPSLOCK",
+    "K_F1",
+    "K_F2",
+    "K_F3",
+    "K_F4",
+    "K_F5",
+    "K_F6",
+    "K_F7",
+    "K_F8",
+    "K_F9",
+    "K_F10",
+    "K_F11",
+    "K_F12",
+    "K_PRINTSCREEN",
+    "K_SCROLLLOCK",
+    "K_PAUSE",
+    "K_INSERT",
+    "K_HOME",
+    "K_PAGEUP",
+    "K_END",
+    "K_PAGEDOWN",
+    "K_RIGHT",
+    "K_LEFT",
+    "K_DOWN",
+    "K_UP",
+    "K_NUMLOCKCLEAR",
+    "K_KP_DIVIDE",
+    "K_KP_MULTIPLY",
+    "K_KP_MINUS",
+    "K_KP_PLUS",
+    "K_KP_ENTER",
+    "K_KP_1",
+    "K_KP_2",
+    "K_KP_3",
+    "K_KP_4",
+    "K_KP_5",
+    "K_KP_6",
+    "K_KP_7",
+    "K_KP_8",
+    "K_KP_9",
+    "K_KP_0",
+    "K_KP_PERIOD",
+    "K_APPLICATION",
+    "K_POWER",
+    "K_KP_EQUALS",
+    "K_F13",
+    "K_F14",
+    "K_F15",
+    "K_F16",
+    "K_F17",
+    "K_F18",
+    "K_F19",
+    "K_F20",
+    "K_F21",
+    "K_F22",
+    "K_F23",
+    "K_F24",
+    "K_EXECUTE",
+    "K_HELP",
+    "K_MENU",
+    "K_SELECT",
+    "K_STOP",
+    "K_AGAIN",
+    "K_UNDO",
+    "K_CUT",
+    "K_COPY",
+    "K_PASTE",
+    "K_FIND",
+    "K_MUTE",
+    "K_VOLUMEUP",
+    "K_VOLUMEDOWN",
+    "K_KP_COMMA",
+    "K_KP_EQUALSAS400",
+    "K_ALTERASE",
+    "K_SYSREQ",
+    "K_CANCEL",
+    "K_CLEAR",
+    "K_PRIOR",
+    "K_RETURN2",
+    "K_SEPARATOR",
+    "K_OUT",
+    "K_OPER",
+    "K_CLEARAGAIN",
+    "K_CRSEL",
+    "K_EXSEL",
+    "K_KP_00",
+    "K_KP_000",
+    "K_THOUSANDSSEPARATOR",
+    "K_DECIMALSEPARATOR",
+    "K_CURRENCYUNIT",
+    "K_CURRENCYSUBUNIT",
+    "K_KP_LEFTPAREN",
+    "K_KP_RIGHTPAREN",
+    "K_KP_LEFTBRACE",
+    "K_KP_RIGHTBRACE",
+    "K_KP_TAB",
+    "K_KP_BACKSPACE",
+    "K_KP_A",
+    "K_KP_B",
+    "K_KP_C",
+    "K_KP_D",
+    "K_KP_E",
+    "K_KP_F",
+    "K_KP_XOR",
+    "K_KP_POWER",
+    "K_KP_PERCENT",
+    "K_KP_LESS",
+    "K_KP_GREATER",
+    "K_KP_AMPERSAND",
+    "K_KP_DBLAMPERSAND",
+    "K_KP_VERTICALBAR",
+    "K_KP_DBLVERTICALBAR",
+    "K_KP_COLON",
+    "K_KP_HASH",
+    "K_KP_SPACE",
+    "K_KP_AT",
+    "K_KP_EXCLAM",
+    "K_KP_MEMSTORE",
+    "K_KP_MEMRECALL",
+    "K_KP_MEMCLEAR",
+    "K_KP_MEMADD",
+    "K_KP_MEMSUBTRACT",
+    "K_KP_MEMMULTIPLY",
+    "K_KP_MEMDIVIDE",
+    "K_KP_PLUSMINUS",
+    "K_KP_CLEAR",
+    "K_KP_CLEARENTRY",
+    "K_KP_BINARY",
+    "K_KP_OCTAL",
+    "K_KP_DECIMAL",
+    "K_KP_HEXADECIMAL",
+    "K_LCTRL",
+    "K_LSHIFT",
+    "K_LALT",
+    "K_LGUI",
+    "K_RCTRL",
+    "K_RSHIFT",
+    "K_RALT",
+    "K_RGUI",
+    "K_MODE",
+    "K_AUDIONEXT",
+    "K_AUDIOPREV",
+    "K_AUDIOSTOP",
+    "K_AUDIOPLAY",
+    "K_AUDIOMUTE",
+    "K_MEDIASELECT",
+    "K_WWW",
+    "K_MAIL",
+    "K_CALCULATOR",
+    "K_COMPUTER",
+    "K_AC_SEARCH",
+    "K_AC_HOME",
+    "K_AC_BACK",
+    "K_AC_FORWARD",
+    "K_AC_STOP",
+    "K_AC_REFRESH",
+    "K_AC_BOOKMARKS",
+    "K_BRIGHTNESSDOWN",
+    "K_BRIGHTNESSUP",
+    "K_DISPLAYSWITCH",
+    "K_KBDILLUMTOGGLE",
+    "K_KBDILLUMDOWN",
+    "K_KBDILLUMUP",
+    "K_EJECT",
+    "K_SLEEP",
+    "KMOD_NONE",
+    "KMOD_LSHIFT",
+    "KMOD_RSHIFT",
+    "KMOD_SHIFT",
+    "KMOD_LCTRL",
+    "KMOD_RCTRL",
+    "KMOD_CTRL",
+    "KMOD_LALT",
+    "KMOD_RALT",
+    "KMOD_ALT",
+    "KMOD_LGUI",
+    "KMOD_RGUI",
+    "KMOD_GUI",
+    "KMOD_NUM",
+    "KMOD_CAPS",
+    "KMOD_MODE",
+    "KMOD_RESERVED",
+    "MOUSEWHEEL_NORMAL",
+    "MOUSEWHEEL_FLIPPED",
+    "MOUSEWHEEL",
+]
